/* $Id: gpilotd-lib.h 120 1998-08-11 21:01:51Z eskil $ */

#include <glib.h>
#include <pi-source.h>
#include <pi-dlp.h>

typedef guint GPilotdRequestHandle;

struct GPilotdHandle;
typedef struct GPilotdHandle GPilotdHandle;

enum GPilotdError {
  GPILOTD_NO_ERROR = 0,
  GPILOTD_UNKNOWN_PILOT,
  GPILOTD_UNKNOWN_USER,
  GPILOTD_MISSING_FILE,
  GPILOTD_NO_ACCESS,
  GPILOTD_NO_HANDLE,
  GPILOTD_NO_DAEMON
};

enum GPilotdSurvival {
  GPILOTD_IMMEDIATE,
  GPILOTD_PERSISTANT
};

enum GPilotdNotification {
  GPILOTD_NOTIFY_CONNECT,
  GPILOTD_NOTIFY_DISCONNECT,
  GPILOTD_NOTIFY_INSTALL,
  GPILOTD_NOTIFY_BACKUP,
  GPILOTD_NOTIFY_CONDUIT,
  GPILOTD_NOTIFY_REQUEST_COMPLETION,
  GPILOTD_NOTIFY_REQUEST_TIMEOUT,
  GPILOTD_NOTIFY_USERINFO_REQUESTED
};


typedef void (*GPilotdNotifyFunc)(gchar *pilotid,
				  gpointer data,
				  enum GPilotdNotification notify_type);

/* URL equal to NULL means default gpilotd protocol and location,
   specifies protocol and locations, allows for different protocol
   implementations, all working with different gpilotd's at once
   :). */

GPilotdHandle *gpilotd_connect(gchar *url);

struct GPilotdHandle {

  void *implementation_data;
};

/** Request operations **/

enum GPilotdError request_install(GPilotdHandle *handle,
				  gchar *pilot,
				  gchar *fname,
				  enum GPilotdSurvival survival,
				  int timeout,
				  GPilotdRequestHandle *request_handle); 
enum GPilotdError request_restore(GPilotdHandle *handle,
				  gchar *pilot,
				  gchar *rdir, /* NULL means default*/
				  enum GPilotdSurvival survival,
				  int timeout,
				  GPilotdRequestHandle *request_handle); 
enum GPilotdError request_conduit(GPilotdHandle *handle,
				  gchar *pilot, 
				  gchar *conduit_name, 
				  enum GPilotdSurvival survival,
				  int timeout,
				  GPilotdRequestHandle *request_handle); 
enum GPilotdError  remove_request(GPilotdHandle *handle,
				  GPilotdRequestHandle rhandle);
#if 0
/* why ? Because you plan to make request handle a structure ?
     If it's just for daemon internal removal purposes, let
     it be removed when completed or when remove_request called */
void destroy_request_handle(GPilotdRequestHandle *rhandle);
#endif

/** Information operations **/

enum GPilotdError get_users(GPilotdHandle *handle, 
			    GList *users);
enum GPilotdError get_conduits(GPilotdHandle *handle, 
			       GList *result);
enum GPilotdError get_conduits_by_pilot(GPilotdHandle *handle, 
					gchar *pilot,
					GList *result);
enum GPilotdError get_pilots_by_name(GPilotdHandle *handle, 
				     gchar *username,
				     GList *result);
enum GPilotdError get_pilots_by_login(GPilotdHandle *handle,
				      gchar *uid, 
				      GList *result);
enum GPilotdError get_pilots(GPilotdHandle *handle,
			     GList *result);
enum GPilotdError get_user_name_by_pilot(GPilotdHandle *handle,
					 gchar *pilot,
					 gchar **result);
enum GPilotdError get_user_login_by_pilot(GPilotdHandle *handle,
					  gchar *pilot,
					  gchar **result);
/** Admin operations **/

  /* when called, the cb function will be called with
     notify_type GPILOTD_NOTIFY_USERINFO_REQUESTED and
     data = a PilotUser structure, unless a 
     GPILOTD_NOTIFY_REQUEST_TIMEOUT is returner */
/* If device != NULL, gpilotd will only respond when said device is used,
   if device == NULL, first pilot synced is user */
enum GPilotdError get_userinfo(GPilotdHandle *handle,
			       gchar *device,
			       GPilotdNotifyFunc cb,
			       int timeout,
			       GPilotdRequestHandle *request_handle); 
enum GPilotdError set_userinfo(GPilotdHandle *handle,
			       gchar *device,
			       struct PilotUser *user,
			       int timeout,
			       GPilotdRequestHandle *request_handle); 

/** Monitor methods **/

enum GPilotdError monitor_start(GPilotdHandle *handle,
				gchar *pilotid,
				GPilotdRequestHandle *request_handle); 
enum GPilotdError monitor_off(GPilotdHandle *handle,
			      GPilotdRequestHandle request_handle); 
enum GPilotdError notify_on(GPilotdHandle *handle,
			    enum GPilotdNotification notification,
			    GPilotdNotifyFunc cb);
enum GPilotdError notify_off(GPilotdHandle *handle,
			     enum GPilotdNotification notification);





