#include "pi-source.h"
#include "pi-socket.h"
#include "pi-file.h"
#include "pi-dlp.h"

#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

#include <gnome.h>
#include <orb/orbit.h>
#include "gpilotd-lib.h"
#include "orbit-daemon-glue.h"

extern GPilotd_Callback glob_cb;
extern GList *the_fds;

int get_pilot_connection(char *device) {
  struct pi_sockaddr addr;
  int sd,ret;
  struct PilotUser U;

  if (!(sd = pi_socket(PI_AF_SLP, PI_SOCK_STREAM, PI_PF_PADP))) {
    perror("pi_socket");
    return  -1;
  }

  addr.pi_family = PI_AF_SLP;
  strcpy(addr.pi_device,device);

  ret = pi_bind(sd, (struct sockaddr*)&addr, sizeof(addr));
  if(ret == -1) {
    perror("pi_bind");
    exit(1);
  }

  ret = pi_listen(sd,1);
  if(ret == -1) {
    perror("pi_listen");
    exit(1);
  }

  sd = pi_accept(sd, 0, 0);
  if(sd == -1) {
    perror("pi_accept");
    exit(1);
  }

  dlp_ReadUserInfo(sd,&U);

  printf("Pilot user %s\n",U.username);
  printf("UserID %ld \n", U.userID );

  return sd;
};

gint listen_for_pilot() {
  int pilot;
  int cnt;
  int max;
  fd_set fdset;

  FD_ZERO(&fdset);
  max = 0;

  for(cnt=0;cnt<g_list_length(the_fds);cnt++) {
    if(g_list_nth(the_fds,cnt)->data!=NULL) {
      int the_fd;
      the_fd = GPOINTER_TO_INT(g_list_nth(the_fds,cnt)->data);
      if(the_fd>max)
	max = the_fd;
      FD_SET(the_fd,&fdset);
      g_message("listening to %d",the_fd);
    }
  }
  pilot = open("/dev/pilot",O_RDONLY);
  if(pilot>max) max = pilot;
  FD_SET(pilot,&fdset);

  select(max+1,&fdset,NULL,NULL,NULL);

  close(pilot);
  if(FD_ISSET(pilot,&fdset)) {
    g_message("selected on pilot\n");
    return 1;
  }
  g_message("selected NOT on pilot, giop_main_iterate...\n");
  giop_main_iterate(0);
  return 0;
}

int main(int argc,char *argv[]) {
  CORBA_Environment ev;
  gnomelib_register_arguments();
  gnomelib_init("gpilotd-X-1");

  gpilotd_corba_register_arguments();
  gpilotd_corba_init();

  while(1) {
    if(listen_for_pilot()) pi_close(get_pilot_connection("/dev/pilot"));
    if(glob_cb) {
      GPilotd_Callback_request_completed(glob_cb,"355",100,&ev);
      CORBA_Object_release(glob_cb, &ev);
    }
  }

  gpilotd_corba_quit();
  
  return 0;
}
