#include <gnome.h>
#include <orb/orbit.h>
#include "orbit-daemon-glue.h"
#include "gpilotd-lib.h"

CORBA_Environment ev;
CORBA_ORB orb;
GPilotd_Callback glob_cb;
GList *the_fds;

static CORBA_unsigned_long
request_install(PortableServer_Servant servant, 
		CORBA_char *pilot_id,
		CORBA_char *filename,
		GPilotd_Survival survival,
		CORBA_unsigned_long timeout,
		CORBA_Environment *ev);
static void
notify_on(PortableServer_Servant servant, 
	  GPilotd_EventType event, 
	  GPilotd_Callback cb, 
	  CORBA_Environment * ev);

PortableServer_ServantBase__epv base_epv = {
  NULL,
  NULL,
  NULL
};

POA_GPilotd_Daemon__epv daemon_epv = { 
  NULL, 
  (gpointer)&request_install,
  (gpointer)&notify_on,
};
POA_GPilotd_Daemon__vepv daemon_vepv = { &base_epv, &daemon_epv };
POA_GPilotd_Daemon daemon_servant = { NULL, &daemon_vepv };

static CORBA_unsigned_long
request_install(PortableServer_Servant servant, 
		CORBA_char *pilot_id,
		CORBA_char *filename,
		GPilotd_Survival survival,
		CORBA_unsigned_long timeout,
		CORBA_Environment *ev)  {
  g_message("pilotid  = %s",pilot_id);
  g_message("filename = %s",filename);
  g_message("survival = %d",survival);
  g_message("timeout  = %d",timeout);
  return 42;
}

static void
notify_on(PortableServer_Servant servant, 
	  GPilotd_EventType event, 
	  GPilotd_Callback cb, 
	  CORBA_Environment * ev) {
  g_message("callback at 0x%xn",&cb);
  glob_cb = CORBA_Object_duplicate(cb,ev);
  //GPilotd_Callback_request_completed(glob_cb,"355",100,ev);
  return;
};

/*************************************************************
  The rest of the glue.
  Mostly ripped from gnome-core/panel/orbit-glue.c
 ************************************************************/
void gpilotd_corba_init(void);

static void orb_add_connection(GIOPConnection *cnx)
{
#ifdef DEBUG
  g_message("check on fd %d\n",GIOP_CONNECTION_GET_FD(cnx));
#endif
  the_fds = g_list_append(the_fds,GINT_TO_POINTER(GIOP_CONNECTION_GET_FD(cnx)));
  cnx->user_data = (gpointer)0;
}

static void orb_remove_connection(GIOPConnection *cnx)
{
#ifdef DEBUG
  g_message("remove check on fd %d\n",GIOP_CONNECTION_GET_FD(cnx));
#endif
  the_fds = g_list_remove(the_fds,GINT_TO_POINTER(GIOP_CONNECTION_GET_FD(cnx)));
}

void
gpilotd_corba_clean_up(void)
{
  char hostname [4096];
  char *name;

  gethostname (hostname, sizeof (hostname));
  if (hostname [0] == 0)
    strcpy (hostname, "unknown-host");


  name = g_strconcat ("/CORBA-servers/gpilotd-", hostname, NULL);
  if(gnome_config_has_section(name))
    gnome_config_clean_section(name);

  gnome_config_sync ();
  g_free (name);
}


void
gpilotd_corba_quit(void)
{
  gpilotd_corba_clean_up();
  CORBA_ORB_shutdown(orb, CORBA_FALSE, &ev);
  /* call something that terminates main loop here */
  /*giop_main_quit();*/
}

void
gpilotd_corba_register_arguments(void)
{
}

void
gpilotd_corba_init(void)
{
  PortableServer_ObjectId objid = {0, sizeof("gpilotd"), "gpilotd" };
  PortableServer_POA the_poa;
  GPilotd_Daemon acc;
  char hostname [4096];
  char *name, *ior,*foo;
  int n;

  n = 0;
  foo = NULL;

  the_fds = g_list_alloc();
  g_message("Initializing CORBA for gpilotd\n");

  CORBA_exception_init(&ev);

  IIOPAddConnectionHandler = orb_add_connection;
  IIOPRemoveConnectionHandler = orb_remove_connection;

  /* It's not like ORBit reads the cmdline anyways, right now */
  orb = CORBA_ORB_init(&n, &foo /* dummy */, "mico-local-orb", &ev);

  POA_GPilotd_Daemon__init(&daemon_servant, &ev);
  g_return_if_fail(ev._major == CORBA_NO_EXCEPTION);

  the_poa = (PortableServer_POA)CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
  g_return_if_fail(ev._major == CORBA_NO_EXCEPTION);

  PortableServer_POAManager_activate(PortableServer_POA__get_the_POAManager(the_poa, &ev), &ev);
  g_return_if_fail(ev._major == CORBA_NO_EXCEPTION);

  PortableServer_POA_activate_object_with_id(the_poa, &objid, &daemon_servant, &ev);
  g_return_if_fail(ev._major == CORBA_NO_EXCEPTION);

  acc = PortableServer_POA_servant_to_reference(the_poa, &daemon_servant, &ev);
  g_return_if_fail(ev._major == CORBA_NO_EXCEPTION);

  ior = CORBA_ORB_object_to_string(orb, acc, &ev);
  
  gethostname (hostname, sizeof (hostname));
  if (hostname [0] == 0)
    strcpy (hostname, "unknown-host");

  g_message("Storing %s\n",ior);
  name = g_strconcat ("/CORBA-servers/gpilotd-", hostname, 
                         "/DISPLAY-", getenv ("DISPLAY"), NULL);
  
  
  gnome_config_set_string (name, ior);
  gnome_config_sync ();

  CORBA_free(ior);

  CORBA_Object_release(acc, &ev);
  g_return_if_fail(ev._major == CORBA_NO_EXCEPTION);

  ORBit_custom_run_setup(orb, &ev);
  g_return_if_fail(ev._major == CORBA_NO_EXCEPTION);
}
