#include "gpilotd-lib.h"
#include <orb/orbit.h>
#include <gnome.h>

GPilotd_Daemon gpilotddaemon;
GPilotd_Callback callback_obj;
CORBA_ORB orb;
CORBA_Environment ev;
char *cb_ior;

static void
cb_pilot_connect(PortableServer_Servant servant,
		 CORBA_char *pilot_id,
		 GPilotd_PilotUser pilotuser,
		 CORBA_Environment *ev) {
  g_message("pilot %s connected",pilot_id);
}

static void
cb_pilot_disconnect(PortableServer_Servant servant,
		    CORBA_char *pilot_id,
		    CORBA_Environment *ev) {
  g_message("pilot %s disconnected",pilot_id);
}

static void
cb_request_completed(PortableServer_Servant servant,
		     CORBA_char *pilot_id,
		     CORBA_unsigned_long id,
		     CORBA_Environment *ev) {
  g_message("pilot %s completed %ul",pilot_id,id);
}

PortableServer_ServantBase__epv base_epv = {
  NULL,
  NULL,
  NULL
};

POA_GPilotd_Callback__epv callback_epv = { 
  NULL, 
  (gpointer)&cb_pilot_connect,
  (gpointer)&cb_pilot_disconnect,
  (gpointer)&cb_request_completed,
};

POA_GPilotd_Callback__vepv callback_vepv = { &base_epv, &callback_epv };
POA_GPilotd_Callback callback_servant = { NULL, &callback_vepv };

int gpilotd_client_reinit_corba(void)
{
  char *name;
  char *iior;
  char hostname [1024];
  int i;
        
  gethostname (hostname, sizeof (hostname));
  if (hostname [0] == 0)
    strcpy (hostname, "unknown-host");

        /*do a 20 second timeout until we get the iior*/
  for(i=0;i<20;i++) {
    name = g_strconcat ("/CORBA-servers/gpilotd-", hostname, 
                           "/ior", NULL);

    g_message("getting ior for %s",name);

    iior = gnome_config_get_string (name);
    g_free (name);

    if(iior)
      break;
    sleep(1);
  }
  if(!iior)
    return 0;

  if(gpilotddaemon)
    CORBA_Object_release(gpilotddaemon, &ev);

  g_message("got %s\n",iior);
  gpilotddaemon = CORBA_ORB_string_to_object(orb, iior, &ev);

  g_return_val_if_fail(gpilotddaemon, 0);
        
  return 1;
}

gint gpilotd_init(int *argc,char **argv[]) {
  PortableServer_ObjectId objid = {0, sizeof("callback"), "callback" };
  PortableServer_POA the_poa;
  GPilotd_Callback acc;
  gint n=0;
  gchar *foo=NULL;

  gnomelib_register_arguments();
  gnomelib_init("gpilotd-X-1");

  g_message("Initializing CORBA for gpilotd client\n");

  CORBA_exception_init(&ev);

  orb = CORBA_ORB_init(&argc, &argv, "orbit-local-orb", &ev); 
  /* orb = CORBA_ORB_init(&n,&foo, "orbit-local-orb", &ev); */
  ORBit_custom_run_setup(orb, &ev);

  /* initialize callback_obj */
  POA_GPilotd_Callback__init(&callback_servant,&ev);
  g_return_val_if_fail(ev._major == CORBA_NO_EXCEPTION,0);

  the_poa = (PortableServer_POA)CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
  g_return_val_if_fail(ev._major == CORBA_NO_EXCEPTION,0);

  PortableServer_POAManager_activate(PortableServer_POA__get_the_POAManager(the_poa, &ev), &ev);
  g_return_val_if_fail(ev._major == CORBA_NO_EXCEPTION,0);

  PortableServer_POA_activate_object_with_id(the_poa, &objid, &callback_servant, &ev);
  g_return_val_if_fail(ev._major == CORBA_NO_EXCEPTION,0);

  callback_obj = PortableServer_POA_servant_to_reference(the_poa, &callback_servant, &ev);
  g_return_val_if_fail(ev._major == CORBA_NO_EXCEPTION,0);

  cb_ior = CORBA_ORB_object_to_string(orb, callback_obj, &ev);
  g_message("callback is %s",cb_ior);
  
  n = gpilotd_client_reinit_corba();

  return n;
}

int gpilotd_monitor_on(gchar *pilot_id) {
  GPilotd_Daemon_monitor_on(gpilotddaemon,pilot_id,&ev);
  g_return_val_if_fail(ev._major == CORBA_NO_EXCEPTION, NULL);
}

int gpilotd_monitor_off(gchar *pilot_id) {
  GPilotd_Daemon_monitor_off(gpilotddaemon,pilot_id,&ev);
  g_return_val_if_fail(ev._major == CORBA_NO_EXCEPTION, NULL);
}

int gpilotd_notify_on(GPilotd_EventType type) {
  GPilotd_Daemon_notify_on(gpilotddaemon,type,cb_ior,&ev);
  g_return_val_if_fail(ev._major == CORBA_NO_EXCEPTION, NULL);
}

int gpilotd_notify_off(GPilotd_EventType type) {
  GPilotd_Daemon_notify_off(gpilotddaemon,type,cb_ior,&ev);
  g_return_val_if_fail(ev._major == CORBA_NO_EXCEPTION, NULL);
}

gint gpilotd_install_file(gchar *pilot_id,
			  gchar *name,
			  GPilotd_Survival survival,
			  int timeout) {
  gint val;
  val = GPilotd_Daemon_request_install(gpilotddaemon,
				       pilot_id,name,
				       survival,timeout,
				       &ev);
  g_return_val_if_fail(ev._major == CORBA_NO_EXCEPTION, NULL);
  g_message("val = %d",val);
  return val;
}

gint gpilotd_set_user_info(gchar *cradle,
			   GPilotd_PilotUser user,
			   GPilotd_Survival survival,
			   int timeout) {
  gint val;
  val = GPilotd_Daemon_set_user_info(gpilotddaemon,
				     &user,cradle,
				     survival,timeout,
				     &ev);
  g_return_val_if_fail(ev._major == CORBA_NO_EXCEPTION, NULL);
  g_message("val = %d",val);
  return val;
}

gint gpilotd_get_user_info(gchar *cradle,
			   GPilotd_Survival survival,
			   int timeout) {
  gint val;
  val = GPilotd_Daemon_get_user_info(gpilotddaemon,
				     cb_ior,
				     cradle,
				     survival,timeout,
				     &ev);
  g_return_val_if_fail(ev._major == CORBA_NO_EXCEPTION, NULL);
  g_message("val = %d",val);
  return val;
}

/***********************************/

int main(int argc,char *argv[]) {

  if(argc<2) {
    g_message("usage : test_client command [options]");
    g_message("        command: install file");
    g_message("        command: getinfo cradle");
    g_message("        command: setinfo cradle name id");
    exit(1);
  }

  gpilotd_init(&argc,&argv);
  gpilotd_monitor_on("p5000");
  gpilotd_notify_on(GPilotd_NOTIFY_CONNECT);
  gpilotd_notify_on(GPilotd_NOTIFY_DISCONNECT);

  if(strcmp(argv[1],"install")==0) {
    if(argc==3)
      gpilotd_install_file("p5000",argv[2],GPilotd_PERSISTANT,0);
    else 
      g_error("filename required");
  } else
    if(strcmp(argv[1],"getinfo")==0) {
      if(argc==3)
	gpilotd_get_user_info(argv[2],GPilotd_PERSISTANT,0);
      else 
	g_error("cradle required");
    } else
      if(strcmp(argv[1],"setinfo")==0) {
	if(argc==5) {
	  GPilotd_PilotUser user;
	  user.userID = atol(argv[4]);
	  user.username = argv[3];
	  gpilotd_set_user_info(argv[2],user,GPilotd_PERSISTANT,0);
	} else 
	  g_error("cradle name id required");
      }

  while(1) { giop_main_iterate(0); };

  CORBA_Object_release(gpilotddaemon, &ev);
  CORBA_Object_release(callback_obj, &ev);
  CORBA_Object_release((CORBA_Object)orb, &ev);
  CORBA_ORB_shutdown(orb, CORBA_FALSE, &ev);

  return 0;
}

