#ifndef _GPILOT_CONDUIT_H_
#define _GPILOT_CONDUIT_H_

#include <glib.h>
#include <pi-dlp.h>

#ifdef __cplusplus
extern "C" {
#endif

  struct GPilotConduit;
  typedef struct GPilotConduit GPilotConduit;
  
  struct GPilotDBInfo {
    struct DBInfo pisock_dbinfo;
    gint pilot_socket;
  };
  
  /* We need to have special methods for the file and backup conduits
     because they also support immediate operations with additional
     parameter such as filenames, and they don't behave like the
     traditional conduits. */

  enum {GPILOT_CONDUIT_STANDARD, 
	GPILOT_CONDUIT_FILE, 
	GPILOT_CONDUIT_BACKUP};

  struct GPilotConduit;
  typedef struct GPilotConduit GPilotConduit;
  
  union GPilotConduitMethods;
  typedef union GPilotConduitMethods GPilotConduitMethods;
  
  union GPilotConduitMethods {
    struct {
      guint32 (*get_creator_id)(GPilotConduit *);
      gchar *(*get_db_name)(GPilotConduit *);
      gint (*copy_to_remote)(GPilotConduit *, GPilotDBInfo *);
      gint (*copy_from_remote)(GPilotConduit *, GPilotDBInfo *);
      gint (*synchronize)(GPilotConduit *, GPilotDBInfo *);
    } standard;
    
    /* Do we need to differentiate DB's and prc's? */
    struct {
      /* If NULL is passed for dbname, the database name is derived
         from the file name if possible, similar usage for install_prc */
      gint (*install_db)(GPilotConduit *, gchar *src_file, gchar *dbname);
      gint (*install_prc)(GPilotConduit *, gchar *src_file, gchar *prcname);
      
      /* Restores all the files in a directory.  Useful with backup_db
	 with a NULL database name, and dest as a file name */
      gint (*restore_directory)(GPilotConduit *, gchar *src_dir);
    } file;
    
    struct {
      /* If NULL is passed for dbname all db's are backed up.  If NULL
         is passed for dest the default destination is used.  If dest
         exists and is a directory standard naming is used to write
         the files, otherwise the path is treated as a file name and
         written */
      gint (*backup_db)(GPilotConduit *, gchar *dest, 
			gchar *dbname); 
      /* Arguments here work similarly to backup_db */
      gint (*backup_prc)(GPilotConduit *, gchar *dest,
			gchar *prcname);
    } backup;
  };
  
  typedef struct GPilotConduitUIMethods {
    gint (*run_configuration_program)(GPilotConduit *,
				      gchar *args[], gchar *env[]);
  }

  struct GPilotConduit {
    int conduit_type;
    GPilotConduitMethods methods;
    GPilotConduitUIMethods ui_methods;
    void *conduit_data; /* Data stored by a conduit */
    void *manager_data; /* Manager data associated with a conduit */
  };
  
  /* Determines if this was the last sync PC or did we synchronize
     with another since the last sync with this one.  return true if
     this was the last sync PC.
     
     NOT YET IMPLEMENTED IN THIS DRAFT */
  extern gboolean gpilot_sync_pc_match(GPilotDBInfo *db); 

#ifdef __cplusplus
	   }
#endif

#endif /* _GPILOT_CONDUIT_H_ */
