#include <glib.h>
#include <gpilot-conduit.h>
#include <pi-socket.h>
#include <pi-sockaddr.h>
#include <pi-source.h>
#include <pi-sync.h>
#include <pi-dlp.h>

#include "manager.h"

/* File and Backup conduits are handled completely separately from the
   other conduits */

typedef gint (*iterate_func)(GPilotConduit *,GPilotDBInfo *);

static gint find_matching_conduit_compare_func(GPilotConduit *a,
					       GPilotDBInfo *b) 
{
  if(a->conduit_type == GPILOT_CONDUIT_STANDARD)
    if((a->methods.standard.get_creator_id() == b->pisock_dbinfo.creator) &&
       !strcmp(a->methods.standard.get_db_name(),b->pisock_dbinfo.name) == 0) {
      return 0;
    }
  return -1;
}

static GPilotConduit *find_matching_conduit(GList *conlist,GPilotDBInfo *dbi)
{
  GList *elem;
  elem=g_list_find_custom(conlist,dbi,*
			  (GCompareFunc)find_matching_conduit_compare_func);
  return elem;
}

static gint iterate_dbs(gint pfd, GList *conlist,  iterate_func f)
{
  GPilotDBInfo info;
  GPilotConduit *match;
  int index;
  int error;
  info.pilot_socket=pfd;
  
  error=0;
  while(1) {
    /* Do we need this? */
    if(dlp_OpenConduit(pfd) < 0) {
      g_error("Unable to open conduit!");
      return -1;
    }

    if(dlp_ReadDBList(pfd, 0, 0x80, index, &info.pisock_dbinfo) < 0)
      break;
    index = info.index+1;
    match=find_matching_conduit(conlist,&info);
    if(!match) continue;
    error=f(match,&info);
  }
  return error;
}

static gint conduit_synchronize(GPilotConduit *c, GPilotDBInfo *dbi)
{
  c->methods.standard.synchronize(c,dbi);
}

static gint conduit_copy_to_remote(GPilotConduit *c, GPilotDBInfo *dbi)
{
  c->methods.standard.copy_to_remote(c,dbi);
}

static gint conduit_copy_from_remote(GPilotConduit *c, GPilotDBInfo *dbi)
{
  c->methods.standard.copy_from_remote(c,dbi);
}

/* Start exported functions for this file */

gint gpilot_synchronize(int pilot_socket,GList *conduit_list)
{
  return iterate_dbs(pilot_socket,conduit_list,conduit_synchronize);
}

gint gpilot_copy_to_remote(int pilot_socket,GList *conduit_list)
{
  return iterate_dbs(pilot_socket,conduit_list,conduit_copy_to_remote);
}

gint gpilot_copy_from_remote(int pilot_socket,GList *conduit_list)
{
  return iterate_dbs(pilot_socket,conduit_list,conduit_copy_from_remote);
}


