#include <stdio.h>
#include <unistd.h>
#include <glib.h>
#include <gnome.h>
#include <gtk/gtk.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <errno.h>
#include <string.h>

#include "gpilot-fileman.h"
#include <multifile.h>

#define APP_ID "gpilot-fileman"

GtkWidget *app;
GtkWidget *multi_file;

/**
 * Called when help/About is chosen from the menu
 **/
void help_about_cb()
{
  GtkWidget *about_box;
  gchar *authors[] = {"Manish Vachharajani <mvachhar@vger.rutgers.edu>",
		      "Neil Vachharajani <nvachhar@pluto.njcc.com>",NULL};
  about_box = gnome_about_new( _("Gnome Pilot File Manager"),
			       GNOME_PILOT_FILEMAN_VERSION,
			       _("Copyright Manish and Neil "
				 "Vachharajani (C) 1998"),
			       authors,
			       _("Manages files and applications on "
				 "your Palm Pilot\n" 
				 "Palm Pilot is a Registered trademark "
				 "of 3com Inc."),
			       NULL);
  gtk_widget_show(about_box);
}

/**
 * Called when a button is pressed on the quit confirmation dialog box
 **/
void quit_dialog_button_cb(GtkWidget *widget, gpointer data)
{
  int but_num = (gint)data;
  switch(but_num) {
  case 0: /* OK */
    gtk_main_quit();
    break;
  case 1:
    break;
  default:
    break;
    
  }
}


/** 
 * Called when the quit menu option is chosen, and when the destroy
 * signal is recieved from the WM.
 **/
void file_quit_cb() {
  GtkWidget *yn;
  if(!prefs->confirm_quit) {
    gtk_main_quit();
    return;
  }
  yn=gnome_message_box_new(N_("Really quit Gnome Pilot File Manager?"),
			   GNOME_MESSAGE_BOX_QUESTION,
			   GNOME_STOCK_BUTTON_OK,
			   GNOME_STOCK_BUTTON_CANCEL,
			   NULL);
  gnome_dialog_button_connect(GNOME_DIALOG(yn),0,
			      GTK_SIGNAL_FUNC(quit_dialog_button_cb),
			      (gpointer)0);
  gnome_dialog_button_connect(GNOME_DIALOG(yn),0,
			      GTK_SIGNAL_FUNC(quit_dialog_button_cb),
			      (gpointer)1);
  gtk_widget_realize(yn);
  gtk_window_set_modal(GTK_WINDOW(yn),TRUE);
  gnome_dialog_set_close(GNOME_DIALOG(yn),TRUE);
  gtk_widget_show(yn);
}

/**
 * Called when we recieve a delete event
 **/
void file_quit_gtk_cb(GtkWidget *w, gpointer *p)
{
  file_quit_cb();
}

/* Structure that is passed to all install functions so they can
   update the GUI, and so the child and parent can synchronize */

typedef struct {
  int sd;
  GtkWidget *pd;
  GtkWidget *pb;
  GtkWidget *label;
  GtkWidget *cf;
  GtkWidget *mb;
  GList *current_file;
  int status;
  gfloat pcnt;
  gfloat inc;
  int read;
  int write;
  pid_t pid;
  int tag;
  int error_fd;
} install_cb_data;


void count_foreach(gpointer data, gpointer userdata) {
  int *i = (int *)userdata;
  (*i)++;
}

/** 
 * Called by the child process doing IO for each file that needs to
 * be installed
 **/
void install_foreach(gpointer data, gpointer userdata) {
  install_cb_data *cbd = (install_cb_data *)userdata;
  int dummy;
  cbd->status=pilot_install(cbd->sd,(gchar *)data);
  if(cbd->status < 0)
    dummy=2;
  else 
    dummy=1;
  write(cbd->write,&dummy,sizeof(int));
}

/**
 * Called to update the progress bar.  Attached to a gdk_input event
 **/
void update_install_progress_cb( gpointer          data,
				 gint              source, 
				 GdkInputCondition condition )
{
  install_cb_data *cbd = (install_cb_data *)data;
  int status;
  int len;
  char *str;
  GtkWidget *mb;

  read(source,&status,sizeof(status));
  if(status > 0) {
    if(status == 2) {
      printf("Showing error\n");
      read(cbd->error_fd,&len,sizeof(len));
      printf("error msg length is %d\n",len);
      str = (char *)alloca(len + 1);
      read(cbd->error_fd,str,len);
      str[len]='\0';
      mb=gnome_message_box_new(str,
			       GNOME_MESSAGE_BOX_ERROR,
			       GNOME_STOCK_BUTTON_OK,NULL); 
      gtk_widget_show(mb);
    } else {
      if(cbd->mb != NULL)
	gnome_dialog_close(GNOME_DIALOG(cbd->mb));
      cbd->mb=NULL;
    }

    if(cbd->current_file)
      cbd->current_file=g_list_next(cbd->current_file);
    else
      g_error(N_("Hmm, this shouldn't happen, not enough filenames\n"));
    if(cbd->current_file)
      gtk_label_set(GTK_LABEL(cbd->cf),cbd->current_file->data);
    else 
      gtk_label_set(GTK_LABEL(cbd->cf),"");
    gtk_progress_bar_update(GTK_PROGRESS_BAR(cbd->pb),cbd->pcnt+cbd->inc);
    cbd->pcnt+=cbd->inc;
  } else {
    if(status < 0) {
      read(cbd->error_fd,&len,sizeof(len));
      str = (char *)alloca(len + 1);
      read(cbd->error_fd,str,len);
      str[len]='\0';
      mb=gnome_message_box_new(str,
			       GNOME_MESSAGE_BOX_ERROR,
			       GNOME_STOCK_BUTTON_OK,NULL); 
      gtk_widget_show(mb);
    }
    waitpid(cbd->pid,NULL,0);
    gnome_dialog_close(GNOME_DIALOG(cbd->pd));
    gdk_input_remove(cbd->tag);
    close(cbd->read);
    close(cbd->error_fd);
    if(cbd->mb) gnome_dialog_close(GNOME_DIALOG(cbd->mb));
    g_free(cbd);
  }
}

/**
 * Called when someone hits cancel in the progress dialog 
 **/
void install_cancel_cb(GtkWidget *widget, gpointer data)
{
  install_cb_data *cbd = (install_cb_data *)data;
  gtk_label_set(GTK_LABEL(cbd->label),N_("Aborting..."));
  gdk_input_remove(cbd->tag);
  if(kill(cbd->pid,SIGTERM) < 0) {
    g_error("gpilot-fileman: %s",strerror(errno));
    return;
  }
  waitpid(cbd->pid,NULL,0);
  close(cbd->read);
  close(cbd->error_fd);
  gnome_dialog_close(GNOME_DIALOG(cbd->pd));
  if(cbd->mb) gnome_dialog_close(GNOME_DIALOG(cbd->mb));
  g_free(cbd);
}

/**
 * Called when someone selects install from the file menu 
 **/
void file_install_cb()
{
  GList *fl;
  int dummy;
  int pilot_socket;
  pid_t pid;
  int p[2],err[2];
  int numfil;
  int i;
  install_cb_data *cbdata;
  GtkWidget *vbox;
  GtkWidget *progress_dialog;
  GtkWidget *label;
  GtkWidget *fn;
  GtkWidget *hbox;
  GtkWidget *pb;
  GtkWidget *mb;
  
  fl=multi_file_get_selected_files(MULTI_FILE(multi_file));
  if(!fl) {
    mb=gnome_message_box_new(_("No files to install!!"),
			     GNOME_MESSAGE_BOX_ERROR,
			     GNOME_STOCK_BUTTON_OK,NULL);
    gtk_widget_show(mb);
    return;
  }
  
  vbox=gtk_vbox_new(FALSE,1);
  progress_dialog=gnome_dialog_new(N_("Install Progress"),
				   GNOME_STOCK_BUTTON_CANCEL,NULL);
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(progress_dialog)->vbox),
		     vbox,TRUE,TRUE,1);
  label=gtk_label_new(N_("Installing... "));
  fn=gtk_label_new("");
  hbox=gtk_hbox_new(FALSE,GNOME_PAD);
  gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,GNOME_PAD);
  gtk_box_pack_start(GTK_BOX(hbox),fn,TRUE,TRUE,GNOME_PAD);
  gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,TRUE,1);
  
  pb=gtk_progress_bar_new();
  gtk_progress_bar_update(GTK_PROGRESS_BAR(pb),0.0);
  gtk_box_pack_start(GTK_BOX(vbox),pb,TRUE,TRUE,1);
  gtk_widget_show(pb);
  gtk_widget_show(label);
  gtk_widget_show(fn);
  gtk_widget_show(hbox);
  gtk_widget_show(vbox);

  cbdata = (install_cb_data *)g_malloc(sizeof(install_cb_data));
  
  cbdata->cf=fn;
  cbdata->pb=pb;
  cbdata->label=label;
  cbdata->status=1;
  cbdata->pcnt=0.0;
  
  if(pipe(p) < 0) {
    gtk_widget_destroy(progress_dialog);
    g_free(cbdata);
    return;
  } 

  if(pipe(err) < 0) {
    gtk_widget_destroy(progress_dialog);
    g_free(cbdata);
    return;
  } 

  cbdata->read=p[0];
  cbdata->write=p[1];
  cbdata->pd=progress_dialog;

  if((pid=fork()) < 0) {
    g_error("Unable to start child, aborting...");
    gtk_widget_destroy(progress_dialog);
    g_free(cbdata);
    return;
  }
  
  if(pid) {
    close(p[1]);
    close(err[1]);
    cbdata->error_fd=err[0];
    numfil=0;
    g_list_foreach(fl,count_foreach,&numfil);
    cbdata->inc=1.0/numfil;
    cbdata->pid=pid;
    mb=gnome_message_box_new(_("Press the Hot Sync button on your pilot now!"),
			     GNOME_MESSAGE_BOX_WARNING,
			     GNOME_STOCK_BUTTON_OK,NULL);
    gtk_widget_show(mb);
    cbdata->mb=mb;
    gtk_window_set_modal(GTK_WINDOW(progress_dialog),TRUE);
    gnome_dialog_button_connect (GNOME_DIALOG(progress_dialog), 0,
				 GTK_SIGNAL_FUNC(install_cancel_cb), cbdata);
    cbdata->current_file=fl;
    gtk_label_set(GTK_LABEL(cbdata->cf),fl->data);
    gtk_widget_show(progress_dialog);
    cbdata->tag=gdk_input_add( p[0],
			       GDK_INPUT_READ,
			       update_install_progress_cb,
			       cbdata);
    return;
  } else {
    close(p[0]);
    close(err[0]);
    close(2);
    dup2(err[1],2);
    install_signal_handlers();
    if((pilot_socket=pilot_connect(prefs->port)) < 0) {
      dummy=-1;
      write(p[1],&dummy,sizeof(int));
      _exit(1);
    } else {
      cbdata->sd=pilot_socket;
      g_list_foreach(fl,install_foreach,cbdata);
      dummy=0;
      write(p[1],&dummy,sizeof(int));
      pilot_disconnect(pilot_socket);
      _exit(0);
    }
  }
}

close_window_cb(GtkWidget *w, GtkWidget *window)
{
  gtk_widget_destroy(window);
}

typedef struct {
  GtkWidget *label;
  GtkWidget *fn;
  GtkWidget *pd;
  GtkWidget *mb;
  int       status_fd;
  int       message_fd; 
  int       tag;
  pid_t     pid;
} backup_cb_data;

void backup_cancel_cb(GtkWidget *w, backup_cb_data *cbd)
{
  gtk_label_set(GTK_LABEL(cbd->label),N_("Aborting..."));
  gdk_input_remove(cbd->tag);
  if(kill(cbd->pid,SIGTERM) < 0) {
    g_error("gpilot-fileman: %s",strerror(errno));
    return;
  }
  waitpid(cbd->pid,NULL,0);
  close(cbd->message_fd);
  close(cbd->status_fd);
  gnome_dialog_close(GNOME_DIALOG(cbd->pd));
  if(cbd->mb) gnome_dialog_close(GNOME_DIALOG(cbd->mb));
  g_free(cbd);
}

void update_backup_progress_cb( gpointer          data,
				 gint              source, 
				 GdkInputCondition condition )
{
  backup_cb_data *cbdata = (backup_cb_data *)data;
  GtkWidget *mb;
  int status;
  
  read(cbdata->status_fd,&status,sizeof(status));
  if(cbdata->mb) {
    gnome_dialog_close(GNOME_DIALOG(cbdata->mb));
    cbdata->mb=NULL;
  }
  if(status) {
    if(status == -2) {
      waitpid(cbdata->pid,NULL,0);
      gnome_dialog_close(GNOME_DIALOG(cbdata->pd));
      close(cbdata->status_fd);
      close(cbdata->message_fd);
      g_free(cbdata);
    } else {
      int mlen;
      gchar *message;

      read(cbdata->message_fd,&mlen,sizeof(mlen));
      message = (gchar *)alloca(mlen + 1);
      read(cbdata->message_fd,message,mlen);
      message[mlen]='\0';
      mb = gnome_message_box_new(message,
				 GNOME_MESSAGE_BOX_ERROR,
				 GNOME_STOCK_BUTTON_OK,NULL);
      gtk_widget_show(mb);
    }
  } else {
    int fnlen;
    gchar *fnt;
    read(cbdata->message_fd,&fnlen,sizeof(fnlen));
    fnt = (gchar *)alloca(fnlen+1);
    read(cbdata->message_fd,fnt,fnlen);
    gtk_label_set(GTK_LABEL(cbdata->fn),fnt);
    gtk_widget_queue_resize(cbdata->fn);
  } 
}

void do_backup_cb(GtkWidget *w, GtkWidget *fsel)
{
  gchar *bdir;
  GtkWidget *progress_dialog,*label,*fn,*hbox;
  GtkWidget *mb;
  int pisock;
  int status_pipe[2], msg_pipe[2];
  pid_t pid;
  backup_cb_data *cbdata;
  struct stat statb;

  bdir = g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fsel)));
  g_return_if_fail(bdir && (bdir[0] != '\0'));  

  /* I should modify the gtk file selection dialog at some 
     point to allow choosing only directories */
  if(stat(bdir,&statb) < 0) {
    if(errno != ENOENT) {
      mb=gnome_message_box_new(_("Unable to open or create chosen name" 
				 " as a directory!"),
			       GNOME_MESSAGE_BOX_ERROR,
			       GNOME_STOCK_BUTTON_OK,NULL);
      gtk_widget_show(mb);
g_free( bdir );
      return;
    }
  }
  if(!S_ISDIR(statb.st_mode)) {
    mb=gnome_message_box_new(_("Chosen backup location is not" 
			       " a directory!"),
			     GNOME_MESSAGE_BOX_ERROR,
			     GNOME_STOCK_BUTTON_OK,NULL);
    gtk_widget_show(mb);
    g_free( bdir );
    return;
  }
  gtk_widget_destroy(fsel);


  progress_dialog=gnome_dialog_new(_("Backup Progress"),
				   GNOME_STOCK_BUTTON_CANCEL,NULL);
  label=gtk_label_new(N_("Backing up... "));
  fn=gtk_label_new("");
  hbox=gtk_hbox_new(FALSE,GNOME_PAD);
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(progress_dialog)->vbox),hbox,TRUE,TRUE,1);
  gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,GNOME_PAD);
  gtk_box_pack_start(GTK_BOX(hbox),fn,TRUE,TRUE,GNOME_PAD);
  
  gtk_widget_show(label);
  gtk_widget_show(fn);
  gtk_widget_show(hbox);

  mb=gnome_message_box_new(_("Press the Hot Sync button on your pilot now!"),
			   GNOME_MESSAGE_BOX_WARNING,
			   GNOME_STOCK_BUTTON_OK,NULL);
  gtk_widget_show(mb);
  
  if(pipe(status_pipe) < 0) {
    gtk_widget_destroy(progress_dialog);
    gtk_widget_destroy(mb);
    mb=gnome_message_box_new(_("Unable to open status pipe"),
			     GNOME_MESSAGE_BOX_ERROR,
			     GNOME_STOCK_BUTTON_OK,NULL);
    gtk_widget_show(mb);
    g_free( bdir );
    return;
  } 

  if(pipe(msg_pipe) < 0) {
    gtk_widget_destroy(progress_dialog);
    gtk_widget_destroy(mb);
    mb=gnome_message_box_new(_("Unable to open message pipe"),
			     GNOME_MESSAGE_BOX_ERROR,
			     GNOME_STOCK_BUTTON_OK,NULL);
    gtk_widget_show(mb);
    g_free( bdir );
    return;
  }

  if((pid = fork()) < 0) {
    gtk_widget_destroy(progress_dialog);
    gtk_widget_destroy(mb);
    mb=gnome_message_box_new(_("Unable to fork"),
			     GNOME_MESSAGE_BOX_ERROR,
			     GNOME_STOCK_BUTTON_OK,NULL);
    gtk_widget_show(mb);
    g_free( bdir );
    return;
  }

  if(pid) {
    close(status_pipe[1]);
    close(msg_pipe[1]);
    cbdata = (backup_cb_data *)g_malloc(sizeof(backup_cb_data));
    cbdata->pid=pid;
    cbdata->fn=fn;
    cbdata->status_fd = status_pipe[0];
    cbdata->message_fd = msg_pipe[0];
    cbdata->pd = progress_dialog;
    cbdata->mb = mb;
    cbdata->label=label;

    gtk_window_set_modal(GTK_WINDOW(progress_dialog),TRUE);
    gnome_dialog_button_connect (GNOME_DIALOG(progress_dialog), 0,
				 GTK_SIGNAL_FUNC(backup_cancel_cb), cbdata);
    cbdata->tag=gdk_input_add( status_pipe[0],
			       GDK_INPUT_READ,
			       update_backup_progress_cb,
			       cbdata);
    gtk_widget_show(progress_dialog);
    return;
  } else {
    int dummy;
    close(status_pipe[0]);
    close(msg_pipe[0]);
    close(2);
    dup2(msg_pipe[1],2);

    install_signal_handlers();
    dummy = -2;
    if((pisock=pilot_connect(prefs->port)) < 0) {
      dummy = -1;
      write(status_pipe[1],&dummy,sizeof(dummy));
      dummy = -2;
      write(status_pipe[1],&dummy,sizeof(dummy));
      _exit(1);
    }
    if(pilot_backup(pisock,status_pipe[1], bdir, 0, 1) < 0) {
      dummy = -1;
      write(status_pipe[1],&dummy,sizeof(dummy));
      dummy=-2;
    }
    pilot_disconnect(pisock);
    write(status_pipe[1],&dummy,sizeof(dummy));
    g_free( bdir );
    printf("Backup done, exiting\n");
    _exit(0);
  }
  
}

/**
 * Called when someone selects backup from the file menu 
 **/
void file_backup_cb()
{
  GtkWidget *file_selection_dialog;
  gchar *homedir;
  
  file_selection_dialog = gtk_file_selection_new(_("Backup Directory"));
  homedir=g_strconcat(getenv("HOME"),"/",NULL);
  if(homedir && homedir[0] != '\0')
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(file_selection_dialog),
				    homedir);
  gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(file_selection_dialog));
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_selection_dialog)->ok_button), 
		     "clicked", GTK_SIGNAL_FUNC(do_backup_cb),file_selection_dialog);
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_selection_dialog)->
				cancel_button), "clicked", 
		     GTK_SIGNAL_FUNC(close_window_cb),file_selection_dialog);
  gtk_widget_show(file_selection_dialog);
}

void create_menus(GtkWidget *app)
{
  GnomeUIInfo helpmenu[] = {
    {GNOME_APP_UI_ITEM,
     N_("About"), N_("About the gpilot-fileman"),
     help_about_cb, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT,
     0, 0, NULL},
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_HELP(APP_ID),
    GNOMEUIINFO_END
  };

  GnomeUIInfo filemenu[] = {
    {GNOME_APP_UI_ITEM,
     N_("Install Files"), N_("Installs selected files to the connected Pilot"),
     file_install_cb,NULL,NULL,
     GNOME_APP_PIXMAP_STOCK,GNOME_STOCK_MENU_BLANK,/*FIXME: reasonable pixmap*/
     0, 0, NULL},

    {GNOME_APP_UI_ITEM,
     N_("Backup Pilot"), N_("Back's up selected files from "
			    "the connected Pilot"),
     file_backup_cb,NULL,NULL,
     GNOME_APP_PIXMAP_STOCK,GNOME_STOCK_MENU_BLANK,/*FIXME: reasonable pixmap*/
     0, 0, NULL},

    {GNOME_APP_UI_ITEM,
     N_("Properties"), N_("Properties for the Gnome Pilot File Manager"),
     preferences_dialog_cb,NULL,NULL,
     GNOME_APP_PIXMAP_STOCK,GNOME_STOCK_MENU_PREF,
     0, 0, NULL},
    
    GNOMEUIINFO_SEPARATOR,

    {GNOME_APP_UI_ITEM,
     N_("Quit"), N_("Quits the application"),
     file_quit_cb, NULL, NULL,
     GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_QUIT,
     0, 0, NULL},
    GNOMEUIINFO_END
  };

  GnomeUIInfo menubar[] = {
    GNOMEUIINFO_SUBTREE(N_("File"), filemenu),
    GNOMEUIINFO_SUBTREE(N_("Help"), helpmenu),
    GNOMEUIINFO_END
  };

  gtk_signal_connect (GTK_OBJECT (app), "delete_event",
		      GTK_SIGNAL_FUNC (file_quit_gtk_cb), NULL);

  gnome_app_create_menus(GNOME_APP(app),menubar);
}

void initialize_gui() 
{
  GtkWidget *multi_file_view;

  app=gnome_app_new(APP_ID, N_("Gnome Pilot File Manager"));
  create_menus(app);
  
  multi_file=multi_file_view=multi_file_new();

  gnome_app_set_contents(GNOME_APP(app),multi_file_view);
  multi_file_set_cwd(MULTI_FILE(multi_file_view), getenv("HOME"));
  gtk_widget_show(multi_file_view);
  gtk_widget_show(app);
}

int main(int argc, char *argv[])
{
  gnome_init(APP_ID, VERSION, argc, argv);
  
  initialize_gui();
  load_preferences();

  gtk_main();
  return 0;
}









