#include <gtk/gtk.h>
#include <multifile.h> /* local, maybe move into gtk? */
#include <gnome.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <string.h>
#include <stdlib.h>


enum {
  FILE_SELECTED_SIGNAL,
  FILE_UNSELECTED_SIGNAL,
  LAST_SIGNAL
};
static gint multi_file_signals[LAST_SIGNAL] = { 0 };

static gint mstrcmp(void *a, void *b)
{
  return strcmp((const char *)a,(const char *)b);
}

typedef struct  {
  MultiFile *w;
  int rownum;
} insert_files_info;

static void insert_files(gchar *fn, insert_files_info *i)
{
  MultiFile *w = i->w;
  gchar *fp;
  gchar *name;
  gchar *c="";
  struct stat stat_buf;
  
  fp=(gchar *)alloca(strlen(fn) + strlen(w->cwd) + 2);
  strcpy(fp,w->cwd);
  strcat(fp,"/");
  strcat(fp,fn);
  
  name=fn;
  if(stat(fp,&stat_buf) >= 0) {
    if(S_ISDIR(stat_buf.st_mode)) {
      name=(gchar *)alloca(strlen(fn) + 2);
      strcpy(name,fn);
      strcat(name,"/");
    }
  } 

  gtk_clist_insert(GTK_CLIST(w->current_file_list),
		   i->rownum, &c);
  gtk_clist_set_text( GTK_CLIST(w->current_file_list),i->rownum,0,name);
  i->rownum++;
}

static void sel_unselected_row_cb(GtkWidget *widget,
				  gint row,
				  gint column,
				  GdkEventButton *event,
				  gpointer data)
{
  MultiFile *w = MULTI_FILE(data);
  w->rcur_pos=-1;
}

static void sel_selected_row_cb(GtkWidget *widget,
				  gint row,
				  gint column,
				  GdkEventButton *event,
				  gpointer data)
{
  MultiFile *w = MULTI_FILE(data);
  w->rcur_pos=row;
}


static void cur_unselected_row_cb(GtkWidget *widget,
				  gint row,
				  gint column,
				  GdkEventButton *event,
				  gpointer data)
{
  MultiFile *w = MULTI_FILE(data);
  if(w->cur_file) g_free(w->cur_file);
  w->cur_file = NULL;
}
     
static void cur_selected_row_cb(GtkWidget *widget,
				gint row,
				gint column,
				GdkEventButton *event,
				gpointer data)
{
  gchar *fn;
  gchar *fp;
  struct stat stat_buf;
  MultiFile *w = MULTI_FILE(data);

  gtk_clist_get_text(GTK_CLIST(widget),row,column,&fn);
  fp=(gchar *)alloca(strlen(fn) + strlen(w->cwd) + 2);
  strcpy(fp,w->cwd);
  strcat(fp,"/");
  strcat(fp,fn);
  if(fp[strlen(fp) - 1] == '/')
    fp[strlen(fp)-1]='\0';
  if(stat(fp,&stat_buf) >= 0) {
    if(S_ISDIR(stat_buf.st_mode)) {
      if(!strcmp(fn,".")) { 
	gtk_clist_unselect_row(GTK_CLIST(widget), row, column);
	return;
      } else if(!strcmp(fn,"../")) {
	char *tmp;
	tmp=strrchr(fp,'/');
	*tmp='\0';
	tmp=strrchr(fp,'/');
	*tmp='\0';
      }
      multi_file_set_cwd(w,fp);
      multi_file_update_file_list(w);
      if(w->cur_file) g_free(w->cur_file);
      w->cur_file = NULL;
    } else {
      if(w->cur_file) g_free(w->cur_file);
      w->cur_file=g_strdup(fp);
    }
  } else {
    if(w->cur_file) g_free(w->cur_file);
    w->cur_file = NULL;
    gtk_clist_unselect_row(GTK_CLIST(widget), row, column);
  }
}

void multi_file_update_file_list(MultiFile *w) 
{
  DIR *cwd;
  struct dirent *d;
  GSList *file_list;
  gchar *fn;
  insert_files_info i;

  gtk_clist_clear(GTK_CLIST(w->current_file_list));
  if((cwd = opendir(w->cwd)) == NULL) {
    g_error("Unable to open directory %s", w->cwd);
    return;
  } 
  
  file_list = NULL;
  while((d=readdir(cwd)) != NULL) {
    fn=g_strdup(d->d_name);
    file_list = g_slist_insert_sorted(file_list,fn,(GCompareFunc)mstrcmp);
  }

  i.w=w;
  i.rownum=0;
  gtk_clist_freeze(GTK_CLIST(w->current_file_list));
  g_slist_foreach(file_list,(GFunc)insert_files,&i);
  gtk_clist_thaw(GTK_CLIST(w->current_file_list));

  g_slist_foreach(file_list,(GFunc)g_free,NULL);
  g_slist_free(file_list);
  closedir(cwd);
}

GtkWidget*
multi_file_new ()
{
  return GTK_WIDGET ( gtk_type_new (multi_file_get_type ()));
}

static void add_cb(GtkWidget *widget,gpointer data)
{
  MultiFile *w = MULTI_FILE(data);
  GList *found;
  gchar *c = "";
  found=g_list_find_custom(w->sel_files,w->cur_file, (GCompareFunc)mstrcmp);
  if(found) return;
  if(w->cur_file == NULL) return;
  gtk_clist_insert(GTK_CLIST(w->selected_file_list),
		   0, &c);
  gtk_clist_set_text( GTK_CLIST(w->selected_file_list),0,0,w->cur_file);
  w->sel_files=g_list_append(w->sel_files,g_strdup(w->cur_file));
  gtk_signal_emit(GTK_OBJECT(w),
		  multi_file_signals[FILE_SELECTED_SIGNAL],w->cur_file);
}

static void remove_cb(GtkWidget *wid,gpointer data)
{
  MultiFile *w = MULTI_FILE(data);
  GList *rem;
  gpointer list_data;
  gchar *text;

  if(w->rcur_pos >= 0) {
    gtk_clist_get_text(GTK_CLIST(w->selected_file_list),w->rcur_pos,0,&text);
    rem=g_list_find_custom(w->sel_files,text,(GCompareFunc)mstrcmp);
    list_data=rem->data;
    w->sel_files=g_list_remove(w->sel_files,list_data);
    gtk_clist_remove(GTK_CLIST(w->selected_file_list),w->rcur_pos);
    gtk_signal_emit(GTK_OBJECT(w),
		    multi_file_signals[FILE_UNSELECTED_SIGNAL],list_data);
    g_free(list_data);
  }
}

static void signal_unselect(gchar *fn, GtkWidget *w)
{
  gtk_signal_emit(GTK_OBJECT(w),multi_file_signals[FILE_UNSELECTED_SIGNAL],
		  fn);
}

static void remove_all_cb(GtkWidget *wid,gpointer data)
{
  MultiFile *w = MULTI_FILE(data);
  gtk_clist_clear(GTK_CLIST(w->selected_file_list));
  g_list_foreach(w->sel_files,(GFunc)signal_unselect,w);
  g_list_foreach(w->sel_files,(GFunc)g_free,NULL);
  g_list_free(w->sel_files);
  w->sel_files=NULL;
}

GList *multi_file_get_selected_files(MultiFile *w)
{
  return w->sel_files;
}

/* Implement me !! */
void multi_file_set_selected_files(MultiFile *w,gchar *names[])
{
}

void multi_file_set_cwd(MultiFile *w, const gchar *str) 
{
  g_return_if_fail(w != NULL);
  g_return_if_fail(str != NULL);
  g_return_if_fail(strlen(str) > 0);
  g_free(w->cwd);
  w->cwd = g_strdup(str);
  gtk_clist_set_column_title(GTK_CLIST(w->current_file_list),
			     0,w->cwd);
  multi_file_update_file_list(w);
}

static void create_gui(GtkWidget *widget)
{
  MultiFile *w = MULTI_FILE(widget);

  GtkWidget *add_button;
  GtkWidget *remove_button;
  GtkWidget *remove_all;
  GtkWidget *vbox, *vbox2;
  GtkWidget *table;
  GtkWidget *sw;

  table=GTK_WIDGET(w);
  gtk_table_resize(GTK_TABLE(table),1,3);
  gtk_table_set_homogeneous(GTK_TABLE(table),FALSE);
  gtk_table_set_row_spacings(GTK_TABLE(table),4);
  gtk_table_set_col_spacings(GTK_TABLE(table),4);


  sw = gtk_scrolled_window_new(NULL,NULL);
  gtk_widget_show(sw);

  w->current_file_list=gtk_clist_new(1);
  gtk_container_add(GTK_CONTAINER(sw),w->current_file_list);
  gtk_table_attach(GTK_TABLE(table),sw,0,1,0,1,
		   GTK_EXPAND | GTK_FILL , 
		   GTK_EXPAND | GTK_FILL , 4, 4);
  gtk_widget_set_usize(w->current_file_list,150,240);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_clist_set_column_title(GTK_CLIST(w->current_file_list),
			     0,w->cwd);
  gtk_clist_column_titles_show(GTK_CLIST(w->current_file_list));
  gtk_widget_show(w->current_file_list);
  gtk_signal_connect(GTK_OBJECT(w->current_file_list),
		     "select_row", GTK_SIGNAL_FUNC(cur_selected_row_cb),
		     (gpointer)w);
  gtk_signal_connect(GTK_OBJECT(w->current_file_list),
		     "unselect_row", GTK_SIGNAL_FUNC(cur_unselected_row_cb),
		     (gpointer)w);


  add_button = gtk_button_new_with_label(N_("Add ->"));
  remove_button = gtk_button_new_with_label(N_("<- Remove"));
  remove_all = gtk_button_new_with_label(N_("<- Remove All"));


  vbox2=gtk_vbox_new(FALSE, 4);
  gtk_table_attach(GTK_TABLE(table),vbox2,1,2,0,1,
		   GTK_SHRINK,
		   GTK_SHRINK,
		   4,4);
  vbox=gtk_vbox_new(FALSE,4);
  gtk_box_pack_start(GTK_BOX(vbox2),vbox,TRUE,FALSE,0);
  gtk_box_pack_start(GTK_BOX(vbox),add_button,FALSE,FALSE,0);
  gtk_box_pack_start(GTK_BOX(vbox),remove_button,FALSE,FALSE,0);
  gtk_box_pack_start(GTK_BOX(vbox),remove_all,FALSE,FALSE,0);

  gtk_widget_show(add_button);
  gtk_widget_show(remove_button);
  gtk_widget_show(remove_all);
  gtk_widget_show(vbox);
  gtk_widget_show(vbox2);


  gtk_signal_connect(GTK_OBJECT(add_button),"clicked",
		     GTK_SIGNAL_FUNC(add_cb), w);
  gtk_signal_connect(GTK_OBJECT(remove_button),"clicked",
		     GTK_SIGNAL_FUNC(remove_cb), w);
  gtk_signal_connect(GTK_OBJECT(remove_all),"clicked",
		     GTK_SIGNAL_FUNC(remove_all_cb), w);
  
  sw = gtk_scrolled_window_new(NULL,NULL);
  gtk_widget_show(sw);

  w->selected_file_list=gtk_clist_new(1);
  gtk_container_add(GTK_CONTAINER(sw),w->selected_file_list);
  gtk_table_attach(GTK_TABLE(table),sw,2,3,0,1,
		   GTK_EXPAND | GTK_FILL ,
		   GTK_EXPAND | GTK_FILL ,
		   4,4);
  gtk_widget_set_usize(w->selected_file_list,150,240);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_AUTOMATIC);
  gtk_clist_set_column_title(GTK_CLIST(w->selected_file_list),
			     0,_("Selected Files"));
  gtk_clist_column_titles_show(GTK_CLIST(w->selected_file_list));
  gtk_widget_show(w->selected_file_list);
  gtk_signal_connect(GTK_OBJECT(w->selected_file_list),
		     "select_row", GTK_SIGNAL_FUNC(sel_selected_row_cb),
		     (gpointer)w);
  gtk_signal_connect(GTK_OBJECT(w->selected_file_list),
		     "unselect_row", GTK_SIGNAL_FUNC(sel_unselected_row_cb),
		     (gpointer)w);

  gtk_widget_show(table);
} 

static void multi_file_init(MultiFile *w)
{
  g_return_if_fail(w != NULL);
 
  w->cwd=g_strdup("/");
  
  create_gui(GTK_WIDGET(w));
  w->sel_files=NULL;
  w->cur_file=NULL;
  w->rcur_pos=-1;
}

static void multi_file_signal_marshaller(GtkObject *object,
					 GtkSignalFunc func,
					 gpointer func_data,
					 GtkArg *args)
{
  MultiFileSelectionSignalFunc signal_func;
  
  signal_func = (MultiFileSelectionSignalFunc)func;
  signal_func(object,GTK_VALUE_POINTER(args[0]),func_data);
}

static void multi_file_class_init(MultiFileClass *class)
{
  GtkObjectClass *object_class;
  
  object_class = (GtkObjectClass*)class;
  
  multi_file_signals[FILE_SELECTED_SIGNAL] = 
    gtk_signal_new ("file_selected",
		    GTK_RUN_FIRST,
		    object_class->type,
		    GTK_SIGNAL_OFFSET (MultiFileClass, file_selected),
		    multi_file_signal_marshaller, 
		    GTK_TYPE_NONE, 
		    1, GTK_TYPE_POINTER);

  multi_file_signals[FILE_UNSELECTED_SIGNAL] = 
    gtk_signal_new ("file_unselected",
		    GTK_RUN_FIRST,
		    object_class->type,
		    GTK_SIGNAL_OFFSET (MultiFileClass, file_unselected),
		    multi_file_signal_marshaller, 
		    GTK_TYPE_NONE, 
		    1, GTK_TYPE_POINTER);
  
  
  gtk_object_class_add_signals (object_class, multi_file_signals, LAST_SIGNAL);
  
  class->file_selected = NULL;
  class->file_unselected = NULL;
}

guint multi_file_get_type(void)
{
  static guint multi_file_type = 0;
  if(!multi_file_type) {
    GtkTypeInfo multi_file_info = {
      "MultiFile",
      sizeof(MultiFile),
      sizeof(MultiFileClass),
      (GtkClassInitFunc)multi_file_class_init,
      (GtkObjectInitFunc)multi_file_init,
      (GtkArgSetFunc)NULL,
      (GtkArgGetFunc)NULL,
      NULL
    };
    
    multi_file_type = gtk_type_unique(gtk_table_get_type(), 
				      &multi_file_info);
  }
  return multi_file_type;
}
