#include <string.h>
#include <gtk/gtk.h>
#include <gnome.h>
#include <glib.h>
#include "gpilot-fileman.h"

typedef struct {
  GtkWidget *confirm_button;
  gboolean confirm_button_state;
  GtkWidget *port_entry;
} GPilotFilemanPrefsWidgets;

FilemanPreferences *prefs;
static FilemanPreferences prefsmem;

/**
 * Called when someone selects "Preferences" from the file menu 
 **/

/**
 * Called when a setting in the preferences dialog changes
 **/
static void preferences_dialog_changed_cb(GtkWidget *widget, gpointer data)
{
  GtkWidget *propbox;
  propbox=GTK_WIDGET(data);
  gnome_property_box_changed(GNOME_PROPERTY_BOX(propbox));
}

/**
 * Called when a button in the preferences dialog is pressed
 **/
void preferences_dialog_button_cb(GtkWidget *widget, gint dummy, 
				  gpointer data) 
{
  /** only the apply button is connected so we assume it was pressed **/
  /** We also assume it is just the General page since no others exist **/
  GPilotFilemanPrefsWidgets *w;
  w=(GPilotFilemanPrefsWidgets *)data;
  g_free(prefs->port);
  prefs->port=g_malloc((strlen(gtk_entry_get_text(GTK_ENTRY(w->port_entry)))+1)
			*sizeof(gchar));
  strcpy(prefs->port,gtk_entry_get_text(GTK_ENTRY(w->port_entry)));
  prefs->confirm_quit = w->confirm_button_state;
  save_preferences();
}

static void preferences_dialog_confirm_state_cb(GtkWidget *widget, 
						gpointer data)
{
  GPilotFilemanPrefsWidgets *w;
  w=(GPilotFilemanPrefsWidgets *)data;
  w->confirm_button_state = !w->confirm_button_state;
}

static void preferences_dialog_free_widgets_cb(GtkWidget *widget, 
					       gpointer data)
{
  free(data);
}


/**
 * Called when Files/Preferences is chosen from the menu 
 **/
void preferences_dialog_cb()
{
  GtkWidget *prefs_dlg;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *entry;
  GtkWidget *label;
  GtkWidget *confirm_button;
  GPilotFilemanPrefsWidgets *prefs_widgets;

  prefs_dlg = gnome_property_box_new();

  vbox = gtk_vbox_new(FALSE,1);
  hbox = gtk_hbox_new(FALSE,GNOME_PAD_SMALL);


  label=gtk_label_new(N_("Pilot Device"));
  gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,GNOME_PAD_SMALL);
  entry=gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(entry), prefs->port);
  gtk_box_pack_start(GTK_BOX(hbox),entry,TRUE,TRUE,GNOME_PAD_SMALL);
  gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,TRUE,1);

  confirm_button=gtk_check_button_new_with_label(N_("Confirm on quit"));
  gtk_box_pack_start(GTK_BOX(vbox),confirm_button,FALSE,FALSE,1);
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(confirm_button),
			      prefs->confirm_quit);

  prefs_widgets = (GPilotFilemanPrefsWidgets *)
    malloc(sizeof(GPilotFilemanPrefsWidgets));
  prefs_widgets->confirm_button_state=prefs->confirm_quit;
  prefs_widgets->confirm_button=confirm_button;
  prefs_widgets->port_entry=entry;


  gtk_signal_connect(GTK_OBJECT(confirm_button), "toggled",
		      GTK_SIGNAL_FUNC(preferences_dialog_confirm_state_cb), 
		      prefs_widgets);
  gtk_signal_connect(GTK_OBJECT(entry), "changed",
		      GTK_SIGNAL_FUNC(preferences_dialog_changed_cb), 
		      prefs_dlg);
  gtk_signal_connect(GTK_OBJECT(confirm_button), "toggled",
		      GTK_SIGNAL_FUNC(preferences_dialog_changed_cb), 
		      prefs_dlg);
  gtk_signal_connect(GTK_OBJECT(prefs_dlg),"apply",
		     GTK_SIGNAL_FUNC(preferences_dialog_button_cb),
		     prefs_widgets);
  gtk_signal_connect(GTK_OBJECT(prefs_dlg),"destroy",
		     GTK_SIGNAL_FUNC(preferences_dialog_free_widgets_cb),
		     prefs_widgets);

  gnome_property_box_append_page(GNOME_PROPERTY_BOX(prefs_dlg),
				 vbox, gtk_label_new(N_("General")));
  gtk_widget_show_all(prefs_dlg);
}

static void set_default_preferences() 
{
  prefs->port = (gchar *)g_malloc(sizeof("/dev/ttyS1"));
  strcpy(prefs->port,"/dev/ttyS1");
  prefs->confirm_quit=TRUE;
}

void load_preferences()
{
  prefs=&prefsmem;
  set_default_preferences(); 
  /** FIXME: should load config file and override defaults **/
}

void save_preferences()
{
  /** FIXME: should save config file **/
}
